!= 地球大気向け放射モデル Ver. 2 modified
!
!= radiation model for the Earth's atmosphere Ver. 2 modified
!
! Authors::   Yoshiyuki O. Takahashi
! Version::   $Id: rad_Earth_V2.f90,v 1.9 2015/01/29 12:06:43 yot Exp $
! Tag Name::  $Name:  $
! Copyright:: Copyright (C) GFD Dennou Club, 2008. All rights reserved.
! License::   See COPYRIGHT[link:../../../COPYRIGHT]
!
module rad_Earth_V2_mod
  !
  != 地球大気向け放射モデル Ver. 2
  !
  != radiation model for the Earth's atmosphere Ver. 1
  !
  ! <b>Note that Japanese and English are described in parallel.</b>
  !
  ! 地球大気向け放射モデル.
  !
  ! This is a radiation model for the Earth's atmospehre.
  !
  ! Radiation in the wavenumber range from    0 to  3000 cm-1 
  ! is calculated in the routine of long wave radiation. 
  ! Radiation in the wavenumber range from 1000 to 57143 cm-1 (0.175 to 10 micron) 
  ! is calculated in the routine of shortwave radiation. 
  !
  !== References
  !
  !
  !== Procedures List
  !
  ! RadEarthV2Flux :: 放射フラックスの計算
  ! ------------          :: ------------
  ! RadEarthV2Flux :: Calculate radiation flux
  !
  !== NAMELIST
  !
  ! NAMELIST#rad_Earth_V2_nml
  !

  ! USE statements
  !

  !
  ! Kind type parameter
  !
  use dc_types, only: DP, &      ! Double precision.
    &                 STRING, &  ! Strings.
    &                 TOKEN      ! Keywords.

  ! メッセージ出力
  ! Message output
  !
  use dc_message, only: MessageNotify

  ! 格子点設定
  ! Grid points settings
  !
  use gridset, only: imax, & ! 経度格子点数.
                             ! Number of grid points in longitude
    &                jmax, & ! 緯度格子点数.
                             ! Number of grid points in latitude
    &                kmax    ! 鉛直層数.
                             ! Number of vertical level

  implicit none

  private

  real(DP), save :: MeanMolWeight
  real(DP), save :: H2OMolWeight
  real(DP), save :: CO2MolWeight
  real(DP), save :: N2OMolWeight
  real(DP), save :: CH4MolWeight

  integer, save     :: IDCloudIceREffMethod
  !
  integer, parameter:: IDCloudIceREffMethodConst = 0
  integer, parameter:: IDCloudIceREffMethodLin   = 1

  real(DP), save :: CloudWatREff
  real(DP), save :: CloudIceREff

!!$  integer , save              :: SWVer
!!$  integer , save              :: LWVer

  ! 公開変数
  ! Public variables
  !
  logical, save :: rad_Earth_V2_inited = .false.
                              ! 初期設定フラグ.
                              ! Initialization flag

  public :: RadEarthV2Flux
!!$  public :: RadEarthV2FluxforNHM
  public :: RadEarthV2Init

  character(*), parameter:: module_name = 'rad_Earth_V2'
                              ! モジュールの名称.
                              ! Module name
  character(*), parameter:: version = &
    & '$Name:  $' // &
    & '$Id: rad_Earth_V2.f90,v 1.9 2015/01/29 12:06:43 yot Exp $'
                              ! モジュールのバージョン
                              ! Module version

  !--------------------------------------------------------------------------------------

contains

  !--------------------------------------------------------------------------------------

  subroutine RadEarthV2Flux(                   &
    & SolarConst, xy_CosSZA,                   &
    & xy_SurfAlbedo,                           & ! (in)
    & xyz_Press, xyr_Press, xyz_Temp,          & ! (in)
    & xyz_QH2OVap, xyz_QH2OLiq, xyz_QH2OSol,   & ! (in)
    & xyz_QCO2, xyz_QO3, xyz_QN2O, xyz_QCH4,   & ! (in)
    & xyz_CloudCover,                          & ! (in)
    & xy_SurfTemp,                             & ! (in)
    & xyr_RadSUwFlux, xyr_RadSDwFlux,          & ! (out)
    & xyr_RadLUwFlux, xyr_RadLDwFlux,          & ! (out)
    & xyra_DelRadLUwFlux, xyra_DelRadLDwFlux   & ! (out)
    & )


    ! USE statements
    !

    real(DP), intent(in ) :: SolarConst
    real(DP), intent(in ) :: xy_CosSZA       (0:imax-1, 1:jmax)
    real(DP), intent(in ) :: xy_SurfAlbedo   (0:imax-1, 1:jmax)
    real(DP), intent(in ) :: xyz_Press       (0:imax-1, 1:jmax, 1:kmax)
    real(DP), intent(in ) :: xyr_Press       (0:imax-1, 1:jmax, 0:kmax)
    real(DP), intent(in ) :: xyz_Temp        (0:imax-1, 1:jmax, 1:kmax)
    real(DP), intent(in ) :: xyz_QH2OVap     (0:imax-1, 1:jmax, 1:kmax)
                              ! $ q $ .   混合比. Mass mixing ratio of constituents (1)
    real(DP), intent(in ) :: xyz_QH2OLiq     (0:imax-1, 1:jmax, 1:kmax)
    real(DP), intent(in ) :: xyz_QH2OSol     (0:imax-1, 1:jmax, 1:kmax)
    real(DP), intent(in ) :: xyz_QCO2        (0:imax-1, 1:jmax, 1:kmax)
    real(DP), intent(in ) :: xyz_QO3         (0:imax-1, 1:jmax, 1:kmax)
    real(DP), intent(in ) :: xyz_QN2O        (0:imax-1, 1:jmax, 1:kmax)
    real(DP), intent(in ) :: xyz_QCH4        (0:imax-1, 1:jmax, 1:kmax)
    real(DP), intent(in ) :: xyz_CloudCover  (0:imax-1, 1:jmax, 1:kmax)
    real(DP), intent(in ) :: xy_SurfTemp     (0:imax-1, 1:jmax)
    real(DP), intent(out) :: xyr_RadSUwFlux   (0:imax-1, 1:jmax, 0:kmax)
    real(DP), intent(out) :: xyr_RadSDwFlux   (0:imax-1, 1:jmax, 0:kmax)
    real(DP), intent(out) :: xyr_RadLUwFlux    (0:imax-1, 1:jmax, 0:kmax)
    real(DP), intent(out) :: xyr_RadLDwFlux    (0:imax-1, 1:jmax, 0:kmax)
    real(DP), intent(out) :: xyra_DelRadLUwFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)
    real(DP), intent(out) :: xyra_DelRadLDwFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)

!!$    real(DP):: xyr_RadLUwFlux    (0:imax-1, 1:jmax, 0:kmax)
!!$    real(DP):: xyr_RadLDwFlux    (0:imax-1, 1:jmax, 0:kmax)
!!$    real(DP):: xyra_DelRadLUwFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)
!!$    real(DP):: xyra_DelRadLDwFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)


    ! Work variables
    !
    real(DP) :: DistFromStarScld
                             ! Distance between the central star and the planet
    real(DP) :: FluxFactor
    real(DP) :: xy_InAngle    (0:imax-1, 1:jmax)
                             ! sec (入射角).
                             ! sec (angle of incidence)

    real(DP) :: VMRCO2
    real(DP) :: VMRN2O
    real(DP) :: VMRCH4

    real(DP) :: xyz_CloudWatREff(0:imax-1, 1:jmax, 1:kmax)
    real(DP) :: xyz_CloudIceREff(0:imax-1, 1:jmax, 1:kmax)

    integer  :: i
    integer  :: j
    integer  :: k


    ! 初期化確認
    ! Initialization check
    !
    if ( .not. rad_Earth_V2_inited ) then
      call MessageNotify( 'E', module_name, 'This module has not been initialized.' )
    end if


    !
    ! Set cloud effective radius
    !
    xyz_CloudWatREff = CloudWatREff
    !
    select case ( IDCloudIceREffMethod )
    case ( IDCloudIceREffMethodConst )
      xyz_CloudIceREff = CloudIceREff
    case ( IDCloudIceREffMethodLin )
      ! This is a simple model mimicing a result of Ou and Liou (1995).
      ! This was used in MRI model probably.
      xyz_CloudIceREff = &
        &   ( 20.0e-6_DP              - 150.0e-6_DP             ) &
        & / ( ( 273.15_DP - 60.0_DP ) - ( 273.15_DP - 20.0_DP ) ) &
        & * ( xyz_Temp                - ( 273.15_DP - 20.0_DP ) ) &
        & + 150.0e-6_DP
      xyz_CloudIceREff = min( max( xyz_CloudIceREff, 20.0e-6_DP ), 150.0e-6_DP )
    end select


    ! 短波入射の計算
    ! Calculate short wave (insolation) incoming radiation
    !
    FluxFactor       = 1.0d0
    DistFromStarScld = 1.0d0
    do i = 0, imax - 1
      do j = 1, jmax
        if ( xy_CosSZA(i,j) > 0.0_DP ) then
          xy_InAngle(i,j) = 1.0_DP / xy_CosSZA(i,j)
        else
          xy_InAngle(i,j) = 0.0_DP
        end if
      end do
    end do


    call RadEarthV2FluxCore( &
      & SolarConst, DistFromStarScld, FluxFactor, xy_InAngle, &
      & xy_SurfAlbedo, &
      & xyr_Press, xyz_Press, &
      & xy_SurfTemp, xyz_Temp, &
      & xyz_QH2OVap, xyz_QH2OLiq, xyz_QH2OSol, &
      & xyz_QCO2, xyz_QO3, xyz_QN2O, xyz_QCH4, &
      & xyz_CloudCover, xyz_CloudWatREff, xyz_CloudIceREff, &
      & xyr_RadSUwFlux, xyr_RadSDwFlux,                          & ! (out)
      & xyr_RadLUwFlux, xyr_RadLDwFlux,                          & ! (out)
      & xyra_DelRadLUwFlux, xyra_DelRadLDwFlux                   & ! (out)
      & )


  end subroutine RadEarthV2Flux

  !--------------------------------------------------------------------------------------

  subroutine RadEarthV2FluxCore( &
    & SolarConst, DistFromStarScld, FluxFactor, xy_InAngle, &
    & xy_SurfAlbedo, &
    & xyr_Press, xyz_Press, &
    & xy_SurfTemp, xyz_Temp, &
    & xyz_QH2OVap, xyz_QH2OLiq, xyz_QH2OSol, &
    & xyz_QCO2, xyz_QO3, xyz_QN2O, xyz_QCH4, &
    & xyz_CloudCover, xyz_CloudWatREff, xyz_CloudIceREff, &
    & xyr_RadSUwFlux, xyr_RadSDwFlux,                          & ! (out)
    & xyr_RadLUwFlux, xyr_RadLDwFlux,                          & ! (out)
    & xyra_DelRadLUwFlux, xyra_DelRadLDwFlux                   & ! (out)
    & )

    ! USE statements
    !

    !
    ! Physical constants settings
    !
    use constants, only: Grav     ! $ g $ [m s-2].
                                  !
                                  ! Gravitational acceleration

    ! 地球大気向け短波放射モデル Ver. 2.7
    ! short wave radiation model for the Earth's atmosphere Ver. 2.7
    !
    use rad_Earth_SW_V2_7, only: RadEarthSWV27Flux

    ! 地球大気向け長波放射モデル Ver. 2.4
    ! long wave radiation model for the Earth's atmosphere Ver. 2.4
    !
    use rad_Earth_LW_V2_4, only : RadEarthLWV24Flux


    real(DP), intent(in ) :: SolarConst
    real(DP), intent(in ) :: DistFromStarScld
    real(DP), intent(in ) :: FluxFactor
    real(DP), intent(in ) :: xy_InAngle      (0:imax-1, 1:jmax)

    real(DP), intent(in ) :: xy_SurfAlbedo   (0:imax-1, 1:jmax)
    real(DP), intent(in ) :: xyr_Press       (0:imax-1, 1:jmax, 0:kmax)
    real(DP), intent(in ) :: xyz_Press       (0:imax-1, 1:jmax, 1:kmax)
    real(DP), intent(in ) :: xy_SurfTemp     (0:imax-1, 1:jmax)
    real(DP), intent(in ) :: xyz_Temp        (0:imax-1, 1:jmax, 1:kmax)
    real(DP), intent(in ) :: xyz_QH2OVap     (0:imax-1, 1:jmax, 1:kmax)
                              ! $ q $ .   混合比. Mass mixing ratio of constituents (1)
    real(DP), intent(in ) :: xyz_QH2OLiq     (0:imax-1, 1:jmax, 1:kmax)
    real(DP), intent(in ) :: xyz_QH2OSol     (0:imax-1, 1:jmax, 1:kmax)
    real(DP), intent(in ) :: xyz_QCO2        (0:imax-1, 1:jmax, 1:kmax)
    real(DP), intent(in ) :: xyz_QO3         (0:imax-1, 1:jmax, 1:kmax)
    real(DP), intent(in ) :: xyz_QN2O        (0:imax-1, 1:jmax, 1:kmax)
    real(DP), intent(in ) :: xyz_QCH4        (0:imax-1, 1:jmax, 1:kmax)
    real(DP), intent(in ) :: xyz_CloudCover  (0:imax-1, 1:jmax, 1:kmax)
    real(DP), intent(in ) :: xyz_CloudWatREff(0:imax-1, 1:jmax, 1:kmax)
    real(DP), intent(in ) :: xyz_CloudIceREff(0:imax-1, 1:jmax, 1:kmax)

    real(DP), intent(out) :: xyr_RadSUwFlux    (0:imax-1, 1:jmax, 0:kmax)
    real(DP), intent(out) :: xyr_RadSDwFlux    (0:imax-1, 1:jmax, 0:kmax)
    real(DP), intent(out) :: xyr_RadLUwFlux    (0:imax-1, 1:jmax, 0:kmax)
    real(DP), intent(out) :: xyr_RadLDwFlux    (0:imax-1, 1:jmax, 0:kmax)
    real(DP), intent(out) :: xyra_DelRadLUwFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)
    real(DP), intent(out) :: xyra_DelRadLDwFlux(0:imax-1, 1:jmax, 0:kmax, 0:1)


    ! Work variables
    !
    real(DP) :: xyz_DelAtmMass   (0:imax-1, 1:jmax, 1:kmax)

    real(DP) :: xyz_DelCO2Mass   (0:imax-1, 1:jmax, 1:kmax)
    real(DP) :: xyz_DelH2OVapMass(0:imax-1, 1:jmax, 1:kmax)
    real(DP) :: xyz_DelH2OLiqMass(0:imax-1, 1:jmax, 1:kmax)
    real(DP) :: xyz_DelH2OSolMass(0:imax-1, 1:jmax, 1:kmax)
    real(DP) :: xyz_DelO3Mass    (0:imax-1, 1:jmax, 1:kmax)
    real(DP) :: xyz_DelN2OMass   (0:imax-1, 1:jmax, 1:kmax)
    real(DP) :: xyz_DelCH4Mass   (0:imax-1, 1:jmax, 1:kmax)

    integer  :: k


    ! 初期化確認
    ! Initialization check
    !
    if ( .not. rad_Earth_V2_inited ) then
      call MessageNotify( 'E', module_name, 'This module has not been initialized.' )
    end if


    do k = 1, kmax
      xyz_DelAtmMass(:,:,k) = ( xyr_Press(:,:,k-1) - xyr_Press(:,:,k ) ) / Grav
    end do

    xyz_DelCO2Mass    = xyz_DelAtmMass * xyz_QCO2
    xyz_DelH2OVapMass = xyz_DelAtmMass * xyz_QH2OVap
    xyz_DelH2OLiqMass = xyz_DelAtmMass * xyz_QH2OLiq
    xyz_DelH2OSolMass = xyz_DelAtmMass * xyz_QH2OSol
    xyz_DelO3Mass     = xyz_DelAtmMass * xyz_QO3
    xyz_DelN2OMass    = xyz_DelAtmMass * xyz_QN2O
    xyz_DelCH4Mass    = xyz_DelAtmMass * xyz_QCH4


    ! 地球大気向け短波放射モデル Ver. 2.7
    ! short wave radiation model for the Earth's atmosphere Ver. 2.7
    !
    call RadEarthSWV27Flux(                                      &
      & SolarConst, xy_InAngle, FluxFactor, DistFromStarScld,    &
      & xy_SurfAlbedo,                                           &
      & xyz_DelAtmMass,                                          &
      & xyz_DelH2OVapMass, xyz_DelH2OLiqMass, xyz_DelH2OSolMass, &
      & xyz_DelO3Mass,                                           &
      & xyz_Press, xyz_Temp,                                     &
      & xyz_CloudCover,                                          &
      & xyz_CloudWatREff, xyz_CloudIceREff,                      &
      & xyr_RadSUwFlux, xyr_RadSDwFlux                           &
      & )

    call RadEarthLWV24Flux(                                      &
      & xyz_DelCO2Mass,                                          &
      & xyz_DelH2OVapMass, xyz_DelH2OLiqMass, xyz_DelH2OSolMass, &
      & xyz_DelO3Mass,                                           &
      & xyz_DelN2OMass, xyz_DelCH4Mass,                          &
      & xyz_Press, xyz_Temp, xy_SurfTemp,                        & ! (in )
      & xyz_QCO2, xyz_QH2OVap,                                   & ! (in )
      & xyz_QN2O, xyz_QCH4,                                      & ! (in )
      & xyz_CloudCover,                                          & ! (in )
      & xyz_CloudWatREff, xyz_CloudIceREff,                      & ! (in )
      & xyr_RadLUwFlux, xyr_RadLDwFlux,                          & ! (out)
      & xyra_DelRadLUwFlux, xyra_DelRadLDwFlux                   & ! (out)
      & )


  end subroutine RadEarthV2FluxCore

  !--------------------------------------------------------------------------------------

  subroutine RadEarthV2Init

    ! ファイル入出力補助
    ! File I/O support
    !
    use dc_iounit, only: FileOpen

    ! ヒストリデータ出力
    ! History data output
    !
    use gtool_historyauto, only: HistoryAutoAddVariable

    ! NAMELIST ファイル入力に関するユーティリティ
    ! Utilities for NAMELIST file input
    !
    use namelist_util, only: namelist_filename, NmlutilMsg, NmlutilAryValid

    ! 地球大気向け短波放射モデル Ver. 2.7
    ! short wave radiation model for the Earth's atmosphere Ver. 2.7
    !
    use rad_Earth_SW_V2_7, only: RadEarthSWV27Init

    ! 地球大気向け長波放射モデル Ver. 2.4
    ! long wave radiation model for the Earth's atmosphere Ver. 2.4
    !
    use rad_Earth_LW_V2_4, only : RadEarthLWV24Init


    ! 宣言文 ; Declaration statements
    !

    character(STRING)   :: CloudIceREffMethod

    integer:: unit_nml        ! NAMELIST ファイルオープン用装置番号.
                              ! Unit number for NAMELIST file open
    integer:: iostat_nml      ! NAMELIST 読み込み時の IOSTAT.
                              ! IOSTAT of NAMELIST read

    ! NAMELIST 変数群
    ! NAMELIST group name
    !
    namelist /rad_Earth_V2_nml/ &
      & CloudIceREffMethod,     &
      & CloudWatREff,           &
      & CloudIceREff

!!$      & SWVer, LWVer
          !
          ! デフォルト値については初期化手続 "rad_Earth_V2#RadEarthV2Init"
          ! のソースコードを参照のこと.
          !
          ! Refer to source codes in the initialization procedure
          ! "rad_Earth_V2#RadEarthV2Init" for the default values.
          !

    if ( rad_Earth_V2_inited ) return


    ! Molecular weights of atmosphere, CO2, H2O, N2O, CH4
    !
    MeanMolWeight = 28.0d-3
    CO2MolWeight  = 44.0d-3
    H2OMolWeight  = 18.0d-3
    N2OMolWeight  = 44.0d-3
    CH4MolWeight  = 16.0d-3


    ! デフォルト値の設定
    ! Default values settings
    !
    CloudIceREffMethod = 'Const'
!!$    CloudIceREffMethod = 'Lin'

    CloudWatREff    = 10.0d-6
    CloudIceREff    = 50.0d-6


!!$    SWVer = 1
!!$    LWVer = 3


    ! NAMELIST の読み込み
    ! NAMELIST is input
    !
    if ( trim(namelist_filename) /= '' ) then
      call FileOpen( unit_nml, &          ! (out)
        & namelist_filename, mode = 'r' ) ! (in)

      rewind( unit_nml )
      read( unit_nml,                     & ! (in)
        & nml = rad_Earth_V2_nml,         & ! (out)
        & iostat = iostat_nml )             ! (out)
      close( unit_nml )

      call NmlutilMsg( iostat_nml, module_name ) ! (in)
    end if


    ! Identification of calculation method of cloud particle effective radius
    !
    call MessageNotify( 'M', module_name, &
      & 'CloudIceREffMethod=<%c>.', &
      & c1 = trim(CloudIceREffMethod) )
    !
    select case ( CloudIceREffMethod )
    case ( 'Const' )
      IDCloudIceREffMethod = IDCloudIceREffMethodConst
    case ( 'Lin' )
      IDCloudIceREffMethod = IDCloudIceREffMethodLin
    case default
      call MessageNotify( 'E', module_name, &
        & 'CloudIceREffMethod=<%c> is not supported.', &
        & c1 = trim(CloudIceREffMethod) )
    end select


    ! Initialization of modules used in this module
    !

    ! 地球大気向け短波放射モデル Ver. 2.7
    ! short wave radiation model for the Earth's atmosphere Ver. 2.7
    !
    call RadEarthSWV27Init

    ! 地球大気向け長波放射モデル Ver. 2.4
    ! long wave radiation model for the Earth's atmosphere Ver. 2.4
    !
    call RadEarthLWV24Init


    ! 印字 ; Print
    !
    call MessageNotify( 'M', module_name, '----- Initialization Messages -----' )
    call MessageNotify( 'M', module_name, '  CloudWatREff      = %f', &
      & d = (/ CloudWatREff /) )
    call MessageNotify( 'M', module_name, '  CloudIceREff      = %f', &
      & d = (/ CloudIceREff /) )
!!$    call MessageNotify( 'M', module_name, 'SWVer = %d', i = (/ SWVer /) )
!!$    call MessageNotify( 'M', module_name, 'LWVer = %d', i = (/ LWVer /) )
    call MessageNotify( 'M', module_name, '-- version = %c', c1 = trim(version) )


    rad_Earth_V2_inited = .true.

  end subroutine RadEarthV2Init

  !--------------------------------------------------------------------------------------

end module rad_Earth_V2_mod
